# Add QT library directory to /etc/ld.so.conf:
if ! grep /usr/lib/qt/lib etc/ld.so.conf 1> /dev/null 2> /dev/null ; then
  echo "/usr/lib/qt/lib" >> etc/ld.so.conf
fi
if [ -x /sbin/ldconfig ]; then
  /sbin/ldconfig 2> /dev/null
fi
( cd usr/bin/32 ; rm -rf moc )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/moc moc )
( cd usr/bin/32 ; rm -rf uic )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/uic uic )
( cd usr/bin/32 ; rm -rf qm2ts )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/qm2ts qm2ts )
( cd usr/bin/32 ; rm -rf qmake )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/qmake qmake )
( cd usr/bin/32 ; rm -rf lrelease )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/lrelease lrelease )
( cd usr/bin/32 ; rm -rf assistant )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/assistant assistant )
( cd usr/bin/32 ; rm -rf lupdate )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/lupdate lupdate )
( cd usr/bin/32 ; rm -rf qtconfig )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/qtconfig qtconfig )
( cd usr/bin/32 ; rm -rf designer )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/designer designer )
( cd usr/bin/32 ; rm -rf linguist )
( cd usr/bin/32 ; ln -sf /usr/lib/qt/bin/linguist linguist )
( cd usr/lib ; rm -rf qt )
( cd usr/lib ; ln -sf qt-3.3.8b qt )
( cd usr/lib/qt-3.3.8b/lib ; rm -rf libqt-mt.so.3 )
( cd usr/lib/qt-3.3.8b/lib ; ln -sf libqt-mt.so.3.3.8 libqt-mt.so.3 )
( cd usr/lib/qt-3.3.8b/lib ; rm -rf libqui.so )
( cd usr/lib/qt-3.3.8b/lib ; ln -sf libqui.so.1.0.0 libqui.so )
( cd usr/lib/qt-3.3.8b/lib ; rm -rf libqui.so.1 )
( cd usr/lib/qt-3.3.8b/lib ; ln -sf libqui.so.1.0.0 libqui.so.1 )
( cd usr/lib/qt-3.3.8b/lib ; rm -rf libqt-mt.so )
( cd usr/lib/qt-3.3.8b/lib ; ln -sf libqt-mt.so.3.3.8 libqt-mt.so )
( cd usr/lib/qt-3.3.8b/lib ; rm -rf libqui.so.1.0 )
( cd usr/lib/qt-3.3.8b/lib ; ln -sf libqui.so.1.0.0 libqui.so.1.0 )
( cd usr/lib/qt-3.3.8b/lib ; rm -rf libqt-mt.so.3.3 )
( cd usr/lib/qt-3.3.8b/lib ; ln -sf libqt-mt.so.3.3.8 libqt-mt.so.3.3 )
( cd usr/lib/qt-3.3.8b/mkspecs ; rm -rf default )
( cd usr/lib/qt-3.3.8b/mkspecs ; ln -sf linux-g++ default )
( cd usr/lib/qt-3.3.8b/mkspecs/linux-g++ ; rm -rf linux-g++ )
( cd usr/lib/qt-3.3.8b/mkspecs/linux-g++ ; ln -sf /tmp/qt-x11-free-3.3.8b/mkspecs/linux-g++ linux-g++ )
( cd usr/lib/pkgconfig ; rm -rf qt-mt.pc )
( cd usr/lib/pkgconfig ; ln -sf /usr/lib/qt-3.3.8b/lib/pkgconfig/qt-mt.pc qt-mt.pc )
