/* Everything */

/* License information */
#define QT_PRODUCT_LICENSEE "Free"
#define QT_PRODUCT_LICENSE  "qt-free"

/* Build key */
#define QT_BUILD_KEY "i686 Linux g++-4 full-config"

/* Machine byte-order */
#define Q_BIG_ENDIAN 4321
#define Q_LITTLE_ENDIAN 1234
#define Q_BYTE_ORDER Q_LITTLE_ENDIAN
/* Compile time features */
#define QT_LARGEFILE_SUPPORT 64
#define QT_POINTER_SIZE 4
#ifndef QT_NO_STYLE_CDE
# define QT_NO_STYLE_CDE
#endif

#ifndef QT_NO_STYLE_COMPACT
# define QT_NO_STYLE_COMPACT
#endif

#ifndef QT_NO_STYLE_MOTIFPLUS
# define QT_NO_STYLE_MOTIFPLUS
#endif

#ifndef QT_NO_STYLE_PLATINUM
# define QT_NO_STYLE_PLATINUM
#endif

#ifndef QT_NO_STYLE_SGI
# define QT_NO_STYLE_SGI
#endif

#ifndef QT_NO_STYLE_WINDOWS
# define QT_NO_STYLE_WINDOWS
#endif

