#ifndef QT_H
#endif /* QT_H */
#ifndef QT_NO_CODECS
#define QT_NO_CODECS
#endif
#define QT_NO_UNICODETABLES
#define QT_NO_IMAGEIO_BMP
#define QT_NO_IMAGEIO_PPM
#define QT_NO_ASYNC_IO
#define QT_NO_ASYNC_IMAGE_IO
/* #define QT_NO_FREETYPE */
#define QT_NO_BDF
//#define QT_NO_FONTDATABASE
#define QT_NO_TRANSLATION
#define QT_NO_DRAGANDDROP
#define QT_NO_CLIPBOARD
#define QT_NO_SOUND
#define QT_NO_PROPERTIES
#define QT_NO_DNS
#define QT_NO_NETWORKPROTOCOL
#define QT_NO_URL
#define QT_NO_COLORNAMES
#define QT_NO_TRANSFORMATIONS
#define QT_NO_PRINTER
#define QT_NO_PICTURE
#define QT_NO_ICONVIEW
#define QT_NO_DIAL
#define QT_NO_WORKSPACE
#define QT_NO_TABLE
#define QT_NO_ACTION
#define QT_NO_STYLE_MOTIF
#define QT_NO_STYLE_PLATINUM
/* #define QT_NO_FILEDIALOG */
#define QT_NO_FONTDIALOG
#define QT_NO_PRINTDIALOG
/* #define QT_NO_COLORDIALOG */
#define QT_NO_INPUTDIALOG
/* #define QT_NO_MESSAGEBOX */
#define QT_NO_PROGRESSDIALOG
/* #define QT_NO_TABDIALOG */
#define QT_NO_WIZARD
#define QT_NO_EFFECTS
